from __future__ import print_function
import unittest
import numpy as np
from chastesweep.util.pscan import Scan


class TestScan(unittest.TestCase):
    def test_error_if_param_val_not_iterable(self):
        pass

    def test_error_if_joint_size_wrong(self):
        pass

    def test_docs_example(self):
        ans = [[0.0, 0.1, 10.0],
               [0.0, 0.1, 10.0],
               [2.5, -0.05, 10.0],
               [2.5, -0.05, 10.0],
               [5.0, -0.2, 10.0],
               [5.0, -0.2, 10.0],
               [7.5, -0.35, 10.0],
               [7.5, -0.35, 10.0],
               [10.0, -0.5, 10.0],
               [10.0, -0.5, 10.0],
               [0.0, 0.1, 15.0],
               [2.5, -0.05, 15.0],
               [5.0, -0.2, 15.0],
               [7.5, -0.35, 15.0],
               [10.0, -0.5, 15.0],
               [0.0, 0.1, 20.0],
               [2.5, -0.05, 20.0],
               [5.0, -0.2, 20.0],
               [7.5, -0.35, 20.0],
               [10.0, -0.5, 20.0], ]
        output = []

        f = lambda a, b, c: output.append([a, b, c])
        p = {}
        p['a'] = np.linspace(0, 10, 5)
        p['b'] = np.linspace(0.1, -0.5, 5)
        p['c'] = np.linspace(10, 20, 3)
        default_count = lambda p: 2
        # None specifies to default to previous value
        big_c_count = lambda p: 1 if p['c'] >= 14 else None

        s = Scan.from_dict(p, joint_lists=[['a', 'b']])
        s.add_count(default_count)  # these will be called
        s.add_count(big_c_count)  # in the order they were added
        s.run_scan(f)  # verify scan validity and run

        num_expanded = len(output)
        num_vars = 3
        self.assertEqual(num_expanded, len(ans), "Length not equal")
        for i in range(num_expanded):
            for j in range(num_vars):
                self.assertAlmostEqual(ans[i][j], output[i][j])



from __future__ import print_function
"""A module whose sole purpose is to facilitate parameter scans via a generic
framework for specifying parameter values to scan.
Combinatorial parameter sweeps, joint parameter sweeps, and advanced options
for how many times to repeat simulations are available.
TODO: implement "scientific" sweeps, where one parameter is varied at a time
from baseline."""
from functools import reduce # for roll-your-own product()
import operator # for operator.mul in map in roll-your-own product()
import collections
import unittest

# class P:
#     """A param for PScan"""
#     def __init__(self, name, values, counts=1):
#         self.name = name
#         self.values = values
#         self.counts = counts

# class PList:
#     """Jointly varying parameters"""
#     def __init__(self, params):
#         self.params = params
#     @classmethod
#     def from_dict(cls, dic, keys=None):
#         if keys is None:
#             keys = dic.keys()
#         return cls([P(key, dic[key]) for key in keys])

def unravel_index(i, shape, order='C'):
    """Rewrite of np.unravel_index that allows shape to have more than 32
    dimensions.

    numpy arrays cannot be of dimension larger than 32. to avoid issues and
    unecessary dependencies we just cut out numpy entirely
    """
    # equivalent to cumulative_sizes = np.cumprod(shape[::-1])[::-1]
    # we product the sizes together in this way because the last dimension
    # "varies fastest" (C-style arrays) by default when ravel is used
    cumulative_sizes = []
    j = 1
    for size in shape[::-1]:
        j *= int(size)
        cumulative_sizes.append(j)
    # the last element is just prod(shape), or the total size of the array
    # being indexed into
    if cumulative_sizes[-1] <= i:
        raise ValueError('Requested index outside of range.')
    # we're going to iterate through, taking interger division then
    # remainder, starting with the largest number and working our way down
    cumulative_sizes = cumulative_sizes[::-1]
    # unravel index algorithm is just like the base change algorithm, but where
    # each digit can be of a "different" base
    indexes = []
    j = i
    for size in cumulative_sizes[1:]:
        indexes.append(j // size)
        j = j % size
    indexes.append(j)
    if order == 'C':
        pass
    elif order == 'F':
        indexes = indexes[::-1]
    else:
        raise ValueError('Expected "C" or "F" for "order" kwarg.')
    return indexes

def get_first(iterable, default=None):
    if iterable:
        for item in iterable:
            return item
    return default

class JointParameterListSizeError(Exception):
    """Raised when two parameters that are meant to vary jointly have a
    different number of values that they are supposed to take."""
    pass

def _check_joint_params(jparam):
    param_names = []
    num_params = []
    for key,val in jparam.items():
        param_names.append(key)
        num_params.append(len(val))
    matches_first = lambda i: num_params[i] != num_params[0]
    bad_inds = filter(matches_first, range(len(num_params)))
    bad_ind = get_first(bad_inds)
    if bad_ind:
        raise JointParameterListSizeError( \
            "Param {} set to vary jointly with {}, but their" \
            "sizes ({},{}) do not match".format(
                param_names[bad_ind], param_names[0],
                num_params[bad_ind], num_params[0]
            )
        )

# this code could replace the two places where we force the comb_params to be
# iterable if we thought this needed to be done by the user to prevent bugs
# for now seems like "just work" is the right choice.

# class ParameterListNotIterableError(Exception):
#     """Raised when a value in the input dictionary (supposed to be a list of
#     parameters to take) is not iterable."""
#     pass

# def _check_comb_param(key, val):
#     if val is None or not isinstance(val, collections.Iterable):
#         raise ParameterListNotIterableError("Param {} asked to take values " \
#                 "{} which should be an iterable list of values (len == 1 is " \
#                 "fine).".format(key, val))

class Scan:
    """Class designed for parameter scanning. Designed under the assumption
    that there are basically three types of "parameter sweeps" that need to be
    done.
    1) run the same parameters many times (e.g. stoch simulation)
    2) vary certain parameters jointly (e.g. (i,j) = (1,2), (2,3), (3,4), ... )
    3) vary parameters combinatorially (e.g. (i,j) = (1,1), (1,2), (2,1), (2,2))
    If a Scan is stopped midway, it will remember its state, allowing you to
    load it back up, change the scan parameters, and continue the run.
    A Scan is *not* useful if your code needs to regularly generate the next
    parameters based on previous simulations.

    If you have several "jointly" varying parameters, each "group" of them will
    interact combinatorially. More abstractly, you should think of each set of
    jointly varying parameters as being the same as a "single" parameter.

    e.g. Suppose we're running a simulation 2 times for each set of (a,b,c),
    except that if 'c' is too big, we run less repeats to save time:

    ```
    >>> import numpy as np
    >>> def f(a,b,c):
    >>>     print('a = ', a, ', b = ', b, ', c = c')
    >>> p['a'] = np.linspace(0, 10, 5)
    >>> p['b'] = np.linspace(0.1, -0.5, 5)
    >>> p['c'] = np.linspace(10, 20, 3)
    >>> default_count = lambda p: 2
    >>> # None specifies to default to previous value
    >>> big_c_count = lambda p: 1 if p['c'] >= 14 else None
    >>>
    >>> s = Scan.from_dict(p, joint_lists=[['a','b']])
    >>> s.add_count(default_count) # these will be called
    >>> s.add_count(big_c_count) # in the order they were added
    >>> s.run_scan(f) # verify scan validity and run

    ```

    will print out

    ```
        0.0 0.1 10.0
        0.0 0.1 10.0
        2.5 -0.05 10.0
        2.5 -0.05 10.0
        5.0 -0.2 10.0
        5.0 -0.2 10.0
        7.5 -0.35 10.0
        7.5 -0.35 10.0
        10.0 -0.5 10.0
        10.0 -0.5 10.0
        0.0 0.1 15.0
        2.5 -0.05 15.0
        5.0 -0.2 15.0
        7.5 -0.35 15.0
        10.0 -0.5 15.0
        0.0 0.1 20.0
        2.5 -0.05 20.0
        5.0 -0.2 20.0
        7.5 -0.35 20.0
        10.0 -0.5 20.0
    ```
    """

    def __init__(self, dic={}, joint_lists=[], default_repeats=1,
                 count_funcs=[]):
# how many times to repeat simulation of a specific parameter by default
        self.default_repeats = default_repeats
# list of functions to iteratively determine how many repeats to actually use
        self.count_funcs = count_funcs
# parameters that need to be combinatorially scanned
        self.comb_params = dict(dic)
# parameters that need to be varied together
        self.joint_params = []
        for jlist in joint_lists:
            # append sub-dictionary with keys from each jlist
            jparam = {key: dic[key] for key in jlist}
            # check self-consistency of joint params before insertion
            _check_joint_params(jparam)
            self.joint_params.append(jparam)
            for key in jlist:
                # remove jointly varying parameters from comb_params as needed
                # as opposed to only adding params to comb_params if they're not
                # in flatten(flatten((joint_lists))
                del self.comb_params[key]
        # make singleton variables iterable
        for key,val in self.comb_params.items():
            if not isinstance(val, collections.Iterable):
                self.comb_params[key] = [self.comb_params[key]]
                #_check_comb_param(key, val)

    def run_scan(self, f):
        """Run f the requested number of times for each set of parameters
        requested."""
        for params in self.params():
            f(**params)

    def params(self):
        """A generator that iterates through all parameters requested the
        correct number of times each. Returns them as a dict with form
        {'param_name': param_value, ... } for use as f(**params)."""
        comb_sizes = []
        comb_keys = []
        for key,val in self.comb_params.items():
            comb_keys.append(key)
            comb_sizes.append(len(val))
        # length of first values array, since lengths should match for sets of
        # jointly varying parameters
        joint_sizes = [len(next(iter(jlist.values()))) for jlist in self.joint_params]
        all_sizes = comb_sizes + joint_sizes
        total_combinations = reduce(operator.mul, all_sizes, 1)
        for i in range(total_combinations):
            sub = unravel_index(i, all_sizes)
            params = {}
            # get the comb_params
            for j in range(len(comb_sizes)):
                key = comb_keys[j]
                val_arr = self.comb_params[key]
                params[key] = val_arr[sub[j]]
            # get the joint_params
            for j in range(len(joint_sizes)):
                subj = j + len(comb_sizes)
                for key,val_arr in self.joint_params[j].items():
                    params[key] = val_arr[sub[subj]]
            # how we have one parameter set, check how many times to repeat it
            num_repeats = self.default_repeats
            for func in self.count_funcs:
                c = func(params)
                if c:
                    num_repeats = c
            for i in range(num_repeats):
                yield params

    def add_count(self, func):
        """Add (a) new function(s) to determine how many times to repeat a parameter
        set. Each function should take a dictionary of values (your parameters)
        and return an integer if it wants to determine how many times to run
        those parameters or None if it wants to defer to the default in that
        case.

        The functions will be called in the order that they are added.
        e.g.
        >>> s.Scan(p)
        >>> default_count = lambda p: 5
        >>> # None specifies to default to previous value
        >>> big_c_count = lambda p: 2 if p['c'] >= 14 else None
        >>> s.add_count(default_count)
        >>> s.add_count(big_c_count)

        """
        self.count_funcs.append(func)

    def add_params(self, p):
        """Add new variables or change values to be used for particular
        parameter names that will not be jointly varied."""
        new_params = p.copy()
        for key,val in new_params.items():
            if not isinstance(val, collections.Iterable):
                new_params[key] = [new_params[key]]
            #_check_comb_param(key, val)
        self.comb_params.update(new_params)

    def add_jparam(self, jparam):
        """Add a new set of parameters that should be varied together. Should
        simply be a dict of "param_name": param_values pairs, where
        all param_values are the same size."""
        _check_joint_params(jparam)
        self.joint_params.append(jparam)

    @classmethod
    def from_dict(cls, dic, joint_lists=[]):
        return cls(dic, joint_lists)




"""Copyright (c) 2005-2019, University of Oxford.
All rights reserved.

University of Oxford means the Chancellor, Masters and Scholars of the
University of Oxford, having an administrative office at Wellington
Square, Oxford OX1 2JD, UK.

This file is part of Chaste.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of the University of Oxford nor the names of its
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
from __future__ import print_function
import unittest
import numpy as np
import json
import os
import shutil

from chastesweep import ParamSweeper
from chastesweep.util.pscan import JointParameterListSizeError


class TestParameterSweeper(unittest.TestCase):


    def test_generate_path(self):

        sweeper = ParamSweeper()

        home_dir = os.path.expanduser("~")

        self.assertEqual(sweeper.get_abs_expanded_path("~/afolder"), os.path.join(home_dir, "afolder"))

        self.assertEqual(None, sweeper.get_abs_expanded_path(None))



    def test_expansion(self):

        p = {}
        p['a'] = np.linspace(0, 10, 5)
        p['b'] = np.linspace(0.1, -0.5, 5)
        p['c'] = [10, 20, 30]

        sweeper = ParamSweeper()

        # First example a is combinatorial with b
        expanded_params = sweeper.expand_parameters(p)

        self.assertEqual(len(expanded_params), 75)

        # Where a and b are joined i.e. no combinatorial expansion
        joint_lists = [['a', 'b']]
        expanded_params = sweeper.expand_parameters(parameters=p, joint_lists=joint_lists)
        self.assertEqual(len(expanded_params), 15)

        # Joins must have the same dimension
        joint_lists_fail = [['a', 'b', 'c']]
        with self.assertRaises(JointParameterListSizeError):
            sweeper.expand_parameters(parameters=p, joint_lists=joint_lists_fail)

        # Change number of default iteration run
        expanded_params = sweeper.expand_parameters(parameters=p, default_repeats=2)
        self.assertEqual(len(expanded_params), 150)

        # Variable iteration, run twice for c < 14 and once for c >= 14
        default_count = lambda p: 2
        big_c_count = lambda p: 1 if p['c'] >= 14 else None

        count_funcs = [default_count, big_c_count]

        expanded_params = sweeper.expand_parameters(parameters=p, count_funcs=count_funcs)
        self.assertEqual(len(expanded_params), 100)

        # Swapping counter, default_counter will overwrite all counts
        count_funcs = [big_c_count, default_count]

        expanded_params = sweeper.expand_parameters(parameters=p, count_funcs=count_funcs)
        self.assertEqual(len(expanded_params), 150)

    def test_batch_generation(self):

        p = {}
        p['a'] = np.linspace(0, 10, 5)
        p['b'] = np.linspace(0.1, -0.5, 5)

        # Feed to param sweeper
        sweeper = ParamSweeper()

        exec_cmd = "chastesweep/test/test_params.sh"
        output_dir = "/tmp/myoutdir_sge"
        scheduler = ParamSweeper.SGE

        if os.path.exists(output_dir):
            shutil.rmtree(output_dir)

        # Test output for SGE
        sweeper.generate_batch_output(output_dir=output_dir,
                                      exec_cmd=exec_cmd,
                                      parameters=p,
                                      batch_params=["-M myemail@mydomain.com", "-m bes"])
        self.assertTrue(os.path.exists(os.path.join(output_dir, sweeper.params_file_name)))
        self.assertTrue(os.path.exists(os.path.join(output_dir, sweeper.sge_batch_file_name)))
        self.assertTrue(os.path.exists(os.path.join(output_dir, sweeper.python_sim_runner_file_name)))

        # Test output json
        expanded_params = sweeper.expand_parameters(p)
        with open(os.path.join(output_dir, sweeper.params_file_name), "r") as out_file:
            out_json = json.load(out_file)
            self.assertEqual(len(expanded_params), len(out_json["params"]))

        # Test output for SLURM
        output_dir = "/tmp/myoutdir_slurm"
        if os.path.exists(output_dir):
            shutil.rmtree(output_dir)

        sweeper.generate_batch_output(output_dir=output_dir,
                                      exec_cmd=exec_cmd,
                                      parameters=p,
                                      scheduler=ParamSweeper.SLURM,
                                      batch_params=["-M myemail@mydomain.com", "-m bes"])

        self.assertTrue(os.path.exists(os.path.join(output_dir, sweeper.params_file_name)))
        self.assertTrue(os.path.exists(os.path.join(output_dir, sweeper.slurm_batch_file_name)))
        self.assertTrue(os.path.exists(os.path.join(output_dir, sweeper.python_sim_runner_file_name)))

    def test_serial_sweep(self):

        p = {}
        p['a'] = np.linspace(0, 10, 5)
        p['b'] = np.linspace(0.1, -0.5, 5)

        # Feed to param sweeper
        sweeper = ParamSweeper()

        exec_cmd = "chastesweep/test/test_params.sh"
        output_dir = "/tmp/serial_sweep"

        if os.path.exists(output_dir):
            shutil.rmtree(output_dir)

        sweeper.perform_serial_sweep(output_dir, exec_cmd, p)

        for i in range(25):
            self.assertTrue(os.path.exists("{}/{}/testout.txt".format(output_dir, i)))

    def test_main_generation(self):

        out_file = "/tmp/test_main.cpp"
        if os.path.exists(out_file):
            os.remove(out_file)

        sweeper = ParamSweeper()
        sweeper.generate_main_cpp(["param1", "param2", "param3"], out_file)

        self.assertTrue(os.path.exists(out_file))



"""Copyright (c) 2005-2019, University of Oxford.
All rights reserved.

University of Oxford means the Chancellor, Masters and Scholars of the
University of Oxford, having an administrative office at Wellington
Square, Oxford OX1 2JD, UK.

This file is part of Chaste.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of the University of Oxford nor the names of its
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
from __future__ import print_function
import os
import json
import subprocess
from jinja2 import Environment, PackageLoader, select_autoescape
from chastesweep.util.pscan import Scan


class ParamSweeper:
    """
    Class to help execute the parameter sweeping process locally and on the cluster (SGE and SLURM support).
    """

    SGE = 0
    SLURM = 1

    def __init__(self):

        self.params_file_name = "params.json"
        self.sge_batch_file_name = "batch.sge.sh"
        self.slurm_batch_file_name = "batch.slurm.sh"
        self.python_sim_runner_file_name = "runsimulation.py"



    def expand_parameters(self, parameters, joint_lists=[], default_repeats=1, count_funcs=[]):
        """
        Generate an expanded list of parameters from the instance variable.
        :param parameters:
        :param joint_lists:
        :param default_repeats:
        :param count_funcs:
        :return: Array of expanded parameters
        """


        scan = Scan(parameters, joint_lists,
                    default_repeats, count_funcs)

        expanded_output = []

        def add_paramdict_to_output(**kwargs):
            iteration_params = {}
            for kw in kwargs:
                iteration_params[kw] = kwargs[kw]

            expanded_output.append(iteration_params)

        scan.run_scan(add_paramdict_to_output)

        return expanded_output

    def generate_batch_output(self, output_dir, exec_cmd, parameters, scheduler=SGE, joint_lists=[], default_repeats=1, count_funcs=[], batch_params=[]):
        """
        Generate output files needed to run parameter sweep in batch mode
        :param output_dir:
        :param exec_cmd:
        :param parameters:
        :param scheduler:
        :param joint_lists:
        :param default_repeats:
        :param count_funcs:
        :param batch_params:
        :return:
        """




        # Expand the paths
        output_dir = self.get_abs_expanded_path(output_dir)
        exec_cmd = self.get_abs_expanded_path(exec_cmd)

        if output_dir is None:
            raise ValueError("Output directory not specified")

        if exec_cmd is None:
            raise ValueError("Must specify an executable command")

        if not os.path.exists(exec_cmd):
            raise ValueError("Could not locate command {}".format(exec_cmd))

        expanded_output = self.expand_parameters(parameters, joint_lists, default_repeats, count_funcs)
        env = Environment(loader=PackageLoader('chastesweep', 'templates'))

        if not os.path.exists(output_dir):
            os.mkdir(output_dir)


        params_output = {"params": expanded_output,
                         "exec_cmd": exec_cmd,
                         "output_dir": output_dir}


        json_output_path = os.path.join(output_dir, self.params_file_name)
        sge_batch_output_path = os.path.join(output_dir, self.sge_batch_file_name)
        slurm_batch_output_path = os.path.join(output_dir, self.slurm_batch_file_name)
        python_sim_runner_output_path = os.path.join(output_dir, self.python_sim_runner_file_name)


        # Output json
        with open(json_output_path, 'w') as json_out_file:
            json.dump(params_output, json_out_file)

        context = {
            "num_tasks": len(expanded_output),
            "exec_cmd": exec_cmd,
            "output_dir": output_dir,
            "batch_params": batch_params
        }

        if scheduler == ParamSweeper.SGE :
            with open(sge_batch_output_path, "w") as sge_batch_file:
                sge_batch_file.write(env.get_template("batch.sge.sh").render(context))
        elif scheduler == ParamSweeper.SLURM:
            with open(slurm_batch_output_path, "w") as slurm_batch_file:
                slurm_batch_file.write(env.get_template("batch.slurm.sh").render(context))
        else:
            raise Exception("Unsupported scheduler {}".format(scheduler))

        with open(python_sim_runner_output_path, "w") as simrunner_file:
            simrunner_file.write(env.get_template("runsimulation.py").render(context))

    def get_abs_expanded_path(self, path):
        if path is None:
            return None

        return os.path.abspath(os.path.expanduser(path))

    def perform_serial_sweep(self, output_dir, exec_cmd, parameters, joint_lists=[], default_repeats=1, count_funcs=[]):
        """
        Runs the sweep serially
        :param output_dir:
        :param exec_cmd:
        :param parameters:
        :param joint_lists:
        :param default_repeats:
        :param count_funcs:
        :return:
        """

        # Expand the paths
        output_dir = self.get_abs_expanded_path(output_dir)
        exec_cmd = self.get_abs_expanded_path(exec_cmd)

        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        # Check path validity
        if output_dir is None:
            raise ValueError("Output directory not specified")

        if exec_cmd is None:
            raise ValueError("Must specify an executable command")

        if not os.path.exists(exec_cmd):
            raise ValueError("Could not locate command {}".format(exec_cmd))

        if not os.path.exists(output_dir):
            os.mkdir(output_dir)

        expanded_output = self.expand_parameters(parameters, joint_lists, default_repeats, count_funcs)
        num_iterations = len(expanded_output)
        for i in range(num_iterations):
            iteration_param = expanded_output[i]

            # Create a folder to store simulation results
            simulation_instance_output_dir = os.path.join(output_dir, str(i))
            if os.path.exists(simulation_instance_output_dir):
                print("Output directory for simulation id {} already exists, aborting".format(i))
            else:
                os.mkdir(simulation_instance_output_dir)

                # Builds the exec command with parameters and run the simulation
                iteration_param_string = ""
                for key, value in iteration_param.items():
                    iteration_param_string = iteration_param_string + " {}={}".format(key, value)
                final_cmd = "{} output_dir={}{}".format(exec_cmd, simulation_instance_output_dir,
                                                        iteration_param_string)

                print("Running simulation ID {}, outputting to {}".format(i, simulation_instance_output_dir))
                subprocess.call(final_cmd, shell=True)



    def generate_main_cpp(self, param_name_list, output_path):
        """
        Generates a main.cpp file from the list of parameter names
        :param param_name_list: A list of parameter names
        :param output_path: Path of the output file
        :return:
        """

        env = Environment(loader=PackageLoader('chastesweep', 'templates'))
        context = {
            "param_names": param_name_list
        }

        with open(output_path, "w") as main_cpp_file:
            main_cpp_file.write(env.get_template("main.cpp").render(context))


        print("*****************************\n")
        print("**** Code output below ******\n\n")
        print(env.get_template("main.cpp").render(context))


import setuptools
with open("README.md", "r") as fh:
    long_description = fh.read()
setuptools.setup(
     name='chastesweep',
     version='0.10',
     scripts=['chastesweep_genmain'],
     author="Twin Karmakharm",
     author_email="t.karmakharm@sheffield.ac.uk",
     description="Parameter Sweeper for Chaste",
     include_package_data=True,
     long_description=long_description,
     long_description_content_type="text/markdown",
     url="https://github.com/twinkarma/chastesweep",
     packages=setuptools.find_packages(),
     install_requires=[
          'numpy', 'jinja2'
      ],
     classifiers=[
         "Programming Language :: Python :: 2",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
     ],
 )


#!/usr/bin/env python
"""Copyright (c) 2005-2019, University of Oxford.
All rights reserved.

University of Oxford means the Chancellor, Masters and Scholars of the
University of Oxford, having an administrative office at Wellington
Square, Oxford OX1 2JD, UK.

This file is part of Chaste.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of the University of Oxford nor the names of its
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
from __future__ import print_function
import os
import sys
import json
import subprocess

if len(sys.argv) < 2:
    print("A simulation ID must be provided as an argument, e.g. : \n runsimulation.py $SGE_TASK_ID")
    sys.exit(1)

try:
    task_id = int(sys.argv[1])
    with open("params.json", "r") as params_file:
        # Load and collect simulation parameters
        exec_params = json.load(params_file)
        exec_cmd = exec_params["exec_cmd"]
        output_dir = exec_params["output_dir"]
        iteration_param = exec_params["params"][task_id - 1]

        # Create a folder to store simulation results
        simulation_instance_output_dir = os.path.join(output_dir, str(task_id))
        if os.path.exists(simulation_instance_output_dir):
            print("Output directory for simulation id {} already exists, aborting".format(task_id))
            sys.exit(1)
        else:
            os.mkdir(simulation_instance_output_dir)

        # Builds the exec command with parameters and run the simulation
        iteration_param_string = ""
        for key, value in iteration_param.items():
            iteration_param_string = iteration_param_string + " {}={}".format(key, value)
        final_cmd = "{} output_dir={}{}".format(exec_cmd, simulation_instance_output_dir, iteration_param_string)

        print("Running simulation ID {}, outputting to {}".format(task_id, simulation_instance_output_dir))
        sys.exit(subprocess.call(final_cmd, shell=True))


except TypeError as e:
    print("Simulation ID must be an integer")
    sys.exit(1)
except IOError as e:
    print("Could not open parameters file")
    sys.exit(1)
except OSError as e:
    print("OS error")
    sys.exit(1)